import { HttpService } from '@nestjs/axios'
import { Injectable } from '@nestjs/common'
import { lastValueFrom } from 'rxjs'

import { RunPromptDto } from './dto'

@Injectable()
export class AgentServerService {
  private readonly baseUrl = 'http://192.168.206.189:8083/api'

  constructor(private readonly httpService: HttpService) {}

  private async makeRequest<T>(
    endpoint: string,
    params?: Record<string, any>
  ): Promise<AgentServerResponseType<T>> {
    const url = `${this.baseUrl}/${endpoint}`
    const responseObservable = await this.httpService.get<
      AgentServerResponseType<T>
    >(url, { params })
    const { data } = await lastValueFrom(responseObservable)
    return data
  }

  async listStaffs() {
    return this.makeRequest<StaffsType>('listStaffs')
  }

  async getStaffProfile(staff_id: string) {
    return this.makeRequest<StaffProfile>('getStaffProfile', { staff_id })
  }

  async getUserProfile(user_id: string) {
    return this.makeRequest<UserProfile>('getUserProfile', { user_id })
  }

  async listUsers(user_name: string, user_union_id: string) {
    return this.makeRequest<UserProfile>('listUsers', {
      user_name,
      user_union_id
    })
  }

  async getDialogueHistory(
    staff_id: string,
    user_id: string,
    recent_minutes: string
  ) {
    return this.makeRequest<DialogueHistory>('getDialogueHistory', {
      staff_id,
      user_id,
      recent_minutes
    })
  }

  async listModels() {
    return this.makeRequest<Model>('listModels')
  }

  async listScenes() {
    return this.makeRequest<Scene>('listScenes')
  }

  async getBasePrompt() {
    return this.makeRequest<BasePrompt>('getBasePrompt')
  }

  async runPrompt(runPromptDto: RunPromptDto) {
    return this.makeRequest<RunPromptDto>('runPrompt', runPromptDto)
  }
}
